# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .general_agent_config_enabled_tools_item import GeneralAgentConfigEnabledToolsItem


class GeneralAgentConfig(UniversalBaseModel):
    """
    Configurable fields for the agent.
    """

    enabled_tools: typing.Optional[typing.List[GeneralAgentConfigEnabledToolsItem]] = None
    knowledge_base_asset_ids: typing.Optional[typing.List[str]] = None
    model: typing.Optional[str] = None
    system_prompt: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
